#include "stdafx.h"
#include "ini.h"
#include <fstream>

#define BUFFER_SIZE 1024

CIniFile::CIniFile(const HINSTANCE hInstance) :
	m_Sections(),
	m_IniFileName(GetDefaultIniFileName(hInstance)),
	m_Fail(FALSE)
{
	Load();
}

CIniFile::CIniFile(const std::wstring IniFileName) :
	m_Sections(),
	m_IniFileName(IniFileName),
	m_Fail(FALSE)
{
	Load();
}

CIniFile::~CIniFile(void)
{
	Release();
}

std::wstring &CIniFile::GetIniFileName(void)
{
	return m_IniFileName;
}

BOOL CIniFile::Load(void)
{
	Release();

	//INIt@Cǂݍ
	Strings data = ReadIniFile();
	if (m_Fail) return m_Fail;

	Strings::iterator it;
	std::wstring section;
	for (it = data.begin(); it != data.end(); it++) {
		if (it->length() <= 0) continue; //s
		if (it->at(0) == _T('#')) continue; //Rg
		if (it->at(0) == _T('[') && it->at(it->length() - 1) == _T(']')) {
			//ZNV
			section = it->substr(1, it->length() - 2);
		} else {
			//l
			Strings::size_type pos = it->find(_T("="));
			if (pos >= 0) {
				std::wstring key = it->substr(0, pos);
				std::wstring value = it->substr(pos + 1);
				AddValue(section, key, value);
			}
		}
	}
	return m_Fail;
}

BOOL CIniFile::Save(void)
{
	std::wstring saveFileName = GetIniFileName() + _T(".save");
	std::wstring tmpFileName = GetIniFileName() + _T(".tmp");

	std::wfstream stream(saveFileName.c_str(), std::ios::out);
	if (stream.fail()) {
		m_Fail = TRUE;
		return FALSE;
	}
	Sections::iterator its = m_Sections.begin();
	for (; its != m_Sections.end(); its++) {
		stream << _T("[") << its->first << _T("]") << std::endl;

		Values::iterator it = its->second.begin();
		for (; it != its->second.end(); it++) {
			stream << it->first << _T("=") << it->second << std::endl;
		}

		stream << std::endl;
	}
	stream.close();

	//init@C̖Oς
	BOOL result = FALSE;
	result = MoveFile(GetIniFileName().c_str(), tmpFileName.c_str());
	//savet@CINIt@CƂ
	result = MoveFile(saveFileName.c_str(), GetIniFileName().c_str());
	if (result) {
		//tmpt@C͏
		DeleteFile(tmpFileName.c_str());
	} else {
		//tmpt@Cɖ߂
		MoveFile(tmpFileName.c_str(), GetIniFileName().c_str());
	}
	
	m_Fail = !result;
	return result;
}

Sections &CIniFile::GetSections(void)
{
	return m_Sections;
}


Values &CIniFile::GetValues(const std::wstring &SectionName, BOOL IsCreate)
{
	try {
		Sections::iterator it = GetSectionPair(SectionName);
		return it->second;
	} catch (wchar_t *msg) {
		if (IsCreate) {
			Values NewValues;
			Sections::iterator now = m_Sections.insert(m_Sections.end(), Section(SectionName, NewValues));
			return now->second;
		}
		throw msg;
	}
}

std::wstring &CIniFile::GetValue(const std::wstring &SectionName, const std::wstring &KeyName)
{
	Values &values = GetValues(SectionName);
	Values::iterator it = GetValuePair(values, KeyName);
	return it->second;
}

std::wstring &CIniFile::GetValue(const std::wstring &SectionName, const std::wstring &KeyName, std::wstring &DefaultValue, BOOL IsSave)
{
	try {
		return GetValue(SectionName, KeyName);
	} catch (wchar_t *) {
		if (IsSave) {
			SetValue(SectionName, KeyName, DefaultValue);
		}
		return DefaultValue;
	}
}

int CIniFile::GetIntValue(const std::wstring &SectionName, const std::wstring &KeyName)
{
	std::wstring value = GetValue(SectionName, KeyName);
	return _ttoi(value.c_str());
}

int CIniFile::GetIntValue(const std::wstring &SectionName, const std::wstring &KeyName, int DefaultValue, BOOL IsSave)
{
	try {
		return GetIntValue(SectionName, KeyName);
	} catch (wchar_t *) {
		if (IsSave) {
			SetIntValue(SectionName, KeyName, DefaultValue);
		}
		return DefaultValue;
	}
}

BOOL CIniFile::SetValue(const std::wstring &SectionName, const std::wstring &KeyName, const std::wstring &ValueData, BOOL WhenNotFoundOnly)
{
	BOOL result = TRUE;

	Values &values = GetValues(SectionName, TRUE);
	try {
		Values::iterator it = GetValuePair(values, KeyName);
		if (WhenNotFoundOnly) {
			result = FALSE;
		} else {
			it->second = ValueData;
		}
	} catch (wchar_t *) {
		values.push_back(Value(KeyName, ValueData));
	}

	return result;
}

BOOL CIniFile::SetIntValue(const std::wstring &SectionName, const std::wstring &KeyName, const int ValueData, BOOL WhenNotFoundOnly)
{
	WCHAR buff[256];
	_itow_s(ValueData, buff, 256, 10);
	return SetValue(SectionName, KeyName, buff, WhenNotFoundOnly);
}

void CIniFile::AddValue(const std::wstring &SectionName, const std::wstring &KeyName, const std::wstring &ValueData)
{
	Values &values = GetValues(SectionName, TRUE);
	values.push_back(Value(KeyName, ValueData));
}

void CIniFile::AddIntValue(const std::wstring &SectionName, const std::wstring &KeyName, const int ValueData)
{
	WCHAR buff[256];
	_itow_s(ValueData, buff, 256, 10);
	AddValue(SectionName, KeyName, buff);
}

BOOL CIniFile::DeleteSection(const std::wstring &SectionName)
{
	try {
		Sections::iterator it = GetSectionPair(SectionName);
		m_Sections.erase(it);
		return TRUE;
	} catch (wchar_t *) {
		//ZNVȂΉȂ
		return FALSE;
	}
}

BOOL CIniFile::DeleteValues(const std::wstring &SectionName)
{
	try {
		Values &values = GetValues(SectionName);
		values.clear();
		return TRUE;
	} catch (wchar_t *) {
		//ZNVȂΉȂ
		return FALSE;
	}
}

BOOL CIniFile::DeleteValue(const std::wstring &SectionName, const std::wstring &KeyName)
{
	try {
		Values &values = GetValues(SectionName);
		Values::iterator it = GetValuePair(values, KeyName);
		values.erase(it);
		return TRUE;
	} catch (wchar_t *) {
		//L[ȂΉȂ
		return FALSE;
	}
}


void CIniFile::Release(void)
{
	m_Sections.clear();
}

std::wstring CIniFile::GetDefaultIniFileName(const HINSTANCE hInstance)
{
	WCHAR ininame[MAX_PATH];
	int len;

	//init@C擾
	len = ::GetModuleFileName(hInstance, ininame, MAX_PATH);
	ininame[len - 3] = 'i';
	ininame[len - 2] = 'n';
	ininame[len - 1] = 'i';
	
	return ininame;
}

CIniFile::Strings CIniFile::ReadIniFile(void)
{
	Strings result;

	std::wfstream stream(GetIniFileName().c_str(), std::ios::in);
	if (stream.fail()) {
		m_Fail = TRUE;
		return result;
	}

	WCHAR buff[BUFFER_SIZE];
	while (!stream.eof()) {
		stream.getline(buff, BUFFER_SIZE);
		if (buff != NULL) result.push_back(buff);
	}

	return result;
}

Sections::iterator CIniFile::GetSectionPair(const std::wstring &SectionName)
{
	Sections::iterator it;
	for (it = m_Sections.begin(); it != m_Sections.end(); it++) {
		if (it->first == SectionName) {
			return it;
		}
	}

	WCHAR msg[512];
	wsprintf(msg, _T("ZNV[%s]܂B"), SectionName.c_str());
	throw msg;
}

Values::iterator CIniFile::GetValuePair(Values &values, const std::wstring &KeyName)
{
	Values::iterator it;
	for (it = values.begin(); it != values.end(); it++) {
		if (it->first == KeyName) {
			return it;
		}
	}

	WCHAR msg[512];
	wsprintf(msg, _T("L[[%s]܂B"), KeyName.c_str());
	throw msg;
}
